package me.totalfreedom.totalfreedommod.command;

import me.totalfreedom.totalfreedommod.config.ConfigEntry;
import me.totalfreedom.totalfreedommod.player.PlayerData;
import me.totalfreedom.totalfreedommod.rank.Rank;
import me.totalfreedom.totalfreedommod.util.FUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import java.util.ArrayList;
import java.util.List;

@CommandPermissions(rank = Rank.OP, source = SourceType.BOTH)
@CommandParameters(description = "Shows the amount of coins you or another player has. Also allows you to give coins to other players.", usage = "/<command> [player] | pay <player> <amount>")
public class Command_coins extends FreedomCommand
{
    @Override
    public boolean run(CommandSender sender, Player playerSender, Command cmd, String commandLabel, String[] args, boolean senderIsConsole)
    {
        if (plugin.sh == null || !ConfigEntry.SHOP_ENABLED.getBoolean())
        {
            msg(Component.text("The shop is currently disabled!", NamedTextColor.RED));
            return true;
        }

        final Component prefix = FUtil.colorizeAsComponent(ConfigEntry.SHOP_PREFIX.getString() + " ");

        switch (args.length)
        {
            // Mode for seeing how many coins the sender has (doesn't work from console)
            case 0 ->
            {
                if (senderIsConsole)
                {
                    msg("When used from the console, you must define a target player.");
                }
                else
                {
                    PlayerData playerData = getData(playerSender);
                    msg(prefix.append(Component.text("You have ", NamedTextColor.GREEN)
                            .append(Component.text(playerData.getCoins(), NamedTextColor.GOLD))
                            .append(Component.text(" coins.", NamedTextColor.GREEN))));
                }
                return true;
            }


            // Mode for seeing how many coins a player has.
            case 1 ->
            {
                Player target = getPlayer(args[0]);

                if (target == null)
                {
                    msg(PLAYER_NOT_FOUND);
                }
                else
                {
                    PlayerData playerData = getData(target);
                    msg(prefix.append(Component.text(target.getName(), NamedTextColor.GOLD)
                            .append(Component.text(" has ", NamedTextColor.GREEN))
                            .append(Component.text(playerData.getCoins(), NamedTextColor.GOLD))
                            .append(Component.text(" coins.", NamedTextColor.GREEN))));
                }
                return true;
            }


            // Mode for paying another player coins
            case 3 ->
            {
                if (args[0].equalsIgnoreCase("pay"))
                {
                    checkPlayer();

                    final Player target = getPlayer(args[1]);
                    final PlayerData senderData = getData(playerSender);

                    int coinsToTransfer;

                    // Processes args[2]
                    try
                    {
                        // Prevents players from trying to be cheeky with negative numbers.
                        coinsToTransfer = Math.max(Math.abs(Integer.parseInt(args[2])), 1);
                    }
                    catch (NumberFormatException ex)
                    {
                        msg(Component.text("Invalid number: " + args[2], NamedTextColor.RED));
                        return true;
                    }

                    // Prevents players from performing transactions they can't afford to do.
                    if (senderData.getCoins() < coinsToTransfer)
                    {
                        msg(Component.text("You don't have enough coins to perform this transaction.", NamedTextColor.RED));
                        return true;
                    }

                    if (target == null)
                    {
                        msg(PLAYER_NOT_FOUND);
                    }
                    else
                    {
                        PlayerData playerData = getData(target);
                        playerData.setCoins(playerData.getCoins() + coinsToTransfer);
                        senderData.setCoins(senderData.getCoins() - coinsToTransfer);

                        boolean plural = coinsToTransfer > 1;

                        msg(target, Component.text(sender.getName(), NamedTextColor.GOLD)
                                .append(Component.text(" has given you ", NamedTextColor.GREEN))
                                .append(Component.text(coinsToTransfer, NamedTextColor.GOLD))
                                .append(Component.text(" coin" + (plural ? "s" : "") + "!", NamedTextColor.GREEN)));

                        msg(target, Component.text("You have given ", NamedTextColor.GREEN)
                                .append(Component.text(coinsToTransfer, NamedTextColor.GOLD))
                                .append(Component.text(" coin" + (plural ? "s" : ""), NamedTextColor.GOLD))
                                .append(Component.text(" to ", NamedTextColor.GREEN))
                                .append(Component.text(target.getName(), NamedTextColor.GOLD))
                                .append(Component.text(".", NamedTextColor.GREEN)));

                        plugin.pl.save(playerData);
                        plugin.pl.save(senderData);
                    }

                    return true;
                }
                return true;
            }
            default ->
            {
                return false;
            }
        }
    }

    @Override
    public List<String> getTabCompleteOptions(CommandSender sender, Command command, String alias, String[] args)
    {
        if (args.length == 1)
        {
            List<String> options = new ArrayList<>(FUtil.getPlayerList());

            options.add("pay");

            return options;
        }

        return FUtil.getPlayerList();
    }
}